import httpx as h
import random
import json

from .template import ImageGet, ImageGetException

class ImageGetMuseum(ImageGet):

    def __init__(self) -> None:
        headers = {
            "AIC-User-Agent": "Eink Art Display (remoll@ethz.ch)"
        }

        super().__init__(
            base_url="https://api.artic.edu/api/v1/artworks/",
            headers=headers
            )


    def get_random_image_ids(self, num=1) -> list[str]:
        post_data = {
            "resources": "artworks",
            "fields": ["id","image_id"],
            "boost": False,
            "limit": num,
            "query": {
                "function_score": {
                    "query": {
                        "bool": {
                            "filter": [
                                {"term": {"is_public_domain": True}},
                                {"exists": {"field": "image_id"}}
                            ]
                        }
                    },
                    "boost_mode": "replace",
                    "random_score": {
                        "field": "id",
                        "seed": f"{random.randint(1, 1000000)}"
                    }
                }
            }
        }

        response = h.post(self.base_url + "search", json=post_data, headers=self.headers)
        if not response.status_code == 200:
            raise ImageGetException("Error in step get_random_image_ids: " + str(response.status_code))

        response = json.loads(response.text)
        images = response["data"]
        ids = [img["id"] for img in images]
        return ids


    def get_image_file(self, image_id: str) -> bytearray:
        url = self.base_url + f"{image_id}"
        print(url)
        response = h.get(url, headers=self.headers)
        if not response.status_code == 200:
            raise ImageGetException("Error in step get_image_file: " + str(response.status_code))
        
        response = json.loads(response.text)
        image_url = response["config"]["iiif_url"] + f"/{response['data']['image_id']}/full/843,/0/default.jpg"
        try:
            image = h.get(image_url).content
        except (h.ConnectError, h.NetworkError, h.RequestError, h.HTTPStatusError) as e:
            raise ImageGetException(f"Error in step get_image_file: {e}")
        return image
